@@ -29,7 +29,7 @@ uiReferences <-
                 </li>
 
                 <li><code>difNLR</code>
-                Hladka, A., Martinkova, P. (2020).
+                Hladka, A., & Martinkova, P. (2020).
                 difNLR: Generalized logistic regression models for DIF and DDF detection.
                 <i>The R Journal, 12</i>(1), 300-323.
                 <a href = "https://doi.org/10.32614/RJ-2020-014", target = "_blank">See online.</a>
@@ -281,7 +281,7 @@ uiReferences <-
 				        London: Chapman and Hall.
                 </li>
 
-                <li>Bartholomew, D. J, Steele, F., Moustaki, I. (Eds.) (2008).
+                <li>Bartholomew, D. J, Steele, F., & Moustaki, I. (Eds.) (2008).
                 <i>Analysis of Multivariate Social Science Data</i> (2nd ed.)
                 CRC Press.
                 </li>
@@ -302,7 +302,7 @@ uiReferences <-
                 </li>
 
                 <li>Cattell, R. B. (1966). The scree test for the number of factors.
-                <i>Multivariate Behavioral Research, 1,</i> 245–276.
+                <i>Multivariate Behavioral Research, 1,</i> 245-276.
                 <a href = "http://dx.doi.org/10.1207/s15327906mbr0102_10",
                 target = "_blank">See online.</a>
                 </li>
@@ -343,12 +343,19 @@ uiReferences <-
                 target = "_blank">See online.</a>
                 </li>
 
-                <li>
-                Horn, J. L. (1965). A rationale and test for the number of factors in factor analysis. <i>Psychometrika, 30,</i> 179–185. <a href="http://dx.doi.org/10.1007/BF02289447", target="_blank">See online.</a>
+                <li>Hladka, A., Martinkova, P., & Magis, D. (2024).
+                Combining item purification and multiple comparison adjustment methods in detection of differential item functioning.
+                <i>Multivariate Behavioral Research 59</i>(1), 46-61.
+                <a href = "https://doi.org/10.1080/00273171.2023.2205393",
+                target = "_blank">See online.</a>
                 </li>
 
                 <li>
-                Kaiser,  H.  F.  (1960).  The  application  of  electronic  computers  to  factor analysis. <i>Educational and Psychological Measurement, 20</i>, 141–151. <a href="http://dx.doi.org/10.1177/001316446002000116", target="_blank">See online.</a>
+                Horn, J. L. (1965). A rationale and test for the number of factors in factor analysis. <i>Psychometrika, 30,</i> 179-185. <a href="http://dx.doi.org/10.1007/BF02289447", target="_blank">See online.</a>
+                </li>
+
+                <li>
+                Kaiser,  H.  F.  (1960).  The  application  of  electronic  computers  to  factor analysis. <i>Educational and Psychological Measurement, 20</i>, 141-151. <a href="http://dx.doi.org/10.1177/001316446002000116", target="_blank">See online.</a>
                 </li>
 
                 <li>Li, H.-H., & Stout, W. (1996).
@@ -410,7 +417,7 @@ uiReferences <-
                 target="_blank">See online.</a>
                 </li>
 
-                <li>Martinkova, P., Drabinova, A., & Potuznikova, E. (2020).
+                <li>Martinkova, P., Hladka, A., & Potuznikova, E. (2020).
                 Is academic tracking related to gains in learning competence?
                 Using propensity score matching and differential item change functioning analysis for better understanding
                 of tracking implications.
@@ -419,6 +426,12 @@ uiReferences <-
                 target = "_blank">See online</a>
                 </li>
 
+                <li>Martinkova, P., & Hladka, A. (2023).
+                <i>Computational Aspects of Psychometric Methods: With R.</i> (1st ed.). Chapman and Hall/CRC.
+                <a href = "https://www.taylorfrancis.com/books/mono/10.1201/9781003054313",
+                target = "_blank">See online</a>
+                </li>
+
                 <li>Masters, G. N. (1982).
                 A Rasch model for partial credit scoring.
                 <i>Psychometrika, 47</i>(2), 149-174.
@@ -429,7 +442,7 @@ uiReferences <-
                 <li>McFarland, J. L., Price, R. M., Wenderoth, M. P., Martinkova, P., Cliff, W., Michael, J., ..., & Wright, A. (2017).
                 Development and validation of the homeostasis concept inventory.
                 <i>CBE-Life Sciences Education, 16</i>(2), ar35.
-                <a href = "http://dx.doi.org/10.1187/cbe.16-10-0305",
+                <a href = "https://doi.org/10.1187/cbe.16-10-0305",
                 target = "_blank">See online.</a>
                 </li>
 
